@@ -116,50 +116,27 @@ Format_description_event::Format_description_event(uint8_t binlog_ver,
         for binlog version 4.
       */
       static uint8_t server_event_header_length[] = {
-          0,
-          QUERY_HEADER_LEN,
-          STOP_HEADER_LEN,
-          ROTATE_HEADER_LEN,
-          INTVAR_HEADER_LEN,
-          0,
+          0, QUERY_HEADER_LEN, STOP_HEADER_LEN, ROTATE_HEADER_LEN,
+          INTVAR_HEADER_LEN, 0,
           /*
             Unused because the code for Slave log event was removed.
             (15th Oct. 2010)
           */
-          0,
-          0,
-          APPEND_BLOCK_HEADER_LEN,
-          0,
-          DELETE_FILE_HEADER_LEN,
-          0,
-          RAND_HEADER_LEN,
-          USER_VAR_HEADER_LEN,
-          FORMAT_DESCRIPTION_HEADER_LEN,
-          XID_HEADER_LEN,
-          BEGIN_LOAD_QUERY_HEADER_LEN,
-          EXECUTE_LOAD_QUERY_HEADER_LEN,
-          TABLE_MAP_HEADER_LEN,
-          0,
-          0,
-          0,
-          ROWS_HEADER_LEN_V1, /* WRITE_ROWS_EVENT_V1*/
-          ROWS_HEADER_LEN_V1, /* UPDATE_ROWS_EVENT_V1*/
-          ROWS_HEADER_LEN_V1, /* DELETE_ROWS_EVENT_V1*/
-          INCIDENT_HEADER_LEN,
-          0, /* HEARTBEAT_LOG_EVENT*/
-          IGNORABLE_HEADER_LEN,
-          IGNORABLE_HEADER_LEN,
-          ROWS_HEADER_LEN_V2,
-          ROWS_HEADER_LEN_V2,
-          ROWS_HEADER_LEN_V2,
+          0, 0, APPEND_BLOCK_HEADER_LEN, 0, DELETE_FILE_HEADER_LEN, 0,
+          RAND_HEADER_LEN, USER_VAR_HEADER_LEN, FORMAT_DESCRIPTION_HEADER_LEN,
+          XID_HEADER_LEN, BEGIN_LOAD_QUERY_HEADER_LEN,
+          EXECUTE_LOAD_QUERY_HEADER_LEN, TABLE_MAP_HEADER_LEN, 0, 0, 0,
+          ROWS_HEADER_LEN_V1,     /* WRITE_ROWS_EVENT_V1*/
+          ROWS_HEADER_LEN_V1,     /* UPDATE_ROWS_EVENT_V1*/
+          ROWS_HEADER_LEN_V1,     /* DELETE_ROWS_EVENT_V1*/
+          INCIDENT_HEADER_LEN, 0, /* HEARTBEAT_LOG_EVENT*/
+          IGNORABLE_HEADER_LEN, IGNORABLE_HEADER_LEN, ROWS_HEADER_LEN_V2,
+          ROWS_HEADER_LEN_V2, ROWS_HEADER_LEN_V2,
           Gtid_event::POST_HEADER_LENGTH, /*GTID_EVENT*/
           Gtid_event::POST_HEADER_LENGTH, /*ANONYMOUS_GTID_EVENT*/
-          IGNORABLE_HEADER_LEN,
-          TRANSACTION_CONTEXT_HEADER_LEN,
-          VIEW_CHANGE_HEADER_LEN,
-          XA_PREPARE_HEADER_LEN,
-          ROWS_HEADER_LEN_V2,
-          TRANSACTION_PAYLOAD_EVENT,
+          IGNORABLE_HEADER_LEN, TRANSACTION_CONTEXT_HEADER_LEN,
+          VIEW_CHANGE_HEADER_LEN, XA_PREPARE_HEADER_LEN, ROWS_HEADER_LEN_V2,
+          TRANSACTION_PAYLOAD_EVENT, 0 /* HEARTBEAT_LOG_EVENT_V2*/
       };
       /*
         Allows us to sanity-check that all events initialized their
@@ -733,6 +710,56 @@ Heartbeat_event::Heartbeat_event(const char *buf,
   BAPI_VOID_RETURN;
 }
 
+Heartbeat_event_v2::Heartbeat_event_v2(const char *buf,
+                                       const Format_description_event *fde)
+    : Binary_log_event(&buf, fde) {
+  BAPI_ENTER("Heartbeat_event_v2::Heartbeat_event_v2(const char*, ...)");
+  READER_TRY_INITIALIZATION;
+  READER_ASSERT_POSITION(fde->common_header_len);
+  if (header()->get_is_valid()) {
+    auto codec = codecs::Factory::build_codec(header()->type_code);
+    // decode the post LOG_EVENT header
+    auto buffer = (const unsigned char *)reader().ptr();
+    size_t buffer_size = reader().available_to_read();
+    auto result = codec->decode(buffer, buffer_size, *this);
+    header()->set_is_valid(result.second == false);
+  }
+  BAPI_VOID_RETURN;
+}
+
+Heartbeat_event_v2::Heartbeat_event_v2()
+    : Binary_log_event(HEARTBEAT_LOG_EVENT_V2) {}
+
+void Heartbeat_event_v2::set_log_filename(const std::string name) {
+  m_log_filename = name;
+}
+void Heartbeat_event_v2::set_log_position(uint64_t position) {
+  m_log_position = position;
+}
+const std::string Heartbeat_event_v2::get_log_filename() const {
+  return m_log_filename;
+}
+uint64_t Heartbeat_event_v2::get_log_position() const { return m_log_position; }
+
+/**
+  This member function returns the len of the event
+
+  @return the event len
+ */
+uint64_t Heartbeat_event_v2::max_encoding_length() {
+  auto max_log_filename_size_old = FN_REFLEN;
+  auto string_terminator_size_old = 1;
+
+  // add TYPE size + LEN size + VALUE size
+  auto max_filename_len_size = 9UL + 9 + FN_REFLEN;
+  auto max_log_position_size = 9UL + 9 + 9;
+
+  // Add new field sizes here ------------------------
+
+  return max_log_filename_size_old + string_terminator_size_old +
+         max_filename_len_size + max_log_position_size;
+}
+
 #ifndef HAVE_MYSYS
 void Rotate_event::print_event_info(std::ostream &info) {
   info << "Binlog Position: " << pos;
@@ -776,6 +803,15 @@ void Xid_event::print_long_info(std::ostream &info) {
   this->print_event_info(info);
 }
 
+void Heartbeat_event_v2::print_event_info(std::ostream &info) {
+  info << "{ 'log filename' : '" << m_log_filename << "', "
+       << "'log_position' : " << m_log_position << " }";
+}
+
+void Heartbeat_event_v2::print_long_info(std::ostream &info) {
+  print_event_info(info);
+}
+
 #endif  // end HAVE_MYSYS
 
 }  // end namespace binary_log
